@@ -1,6 +1,6 @@
 // Iostreams base classes -*- C++ -*-
 
-// Copyright (C) 1997-1999 Cygnus Solutions
+// Copyright (C) 1997-1999 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -163,48 +163,46 @@ namespace std {
     // 27.4.2.1.2  Type ios_base::fmtflags
     typedef _Ios_Fmtflags fmtflags;
     // 27.4.2.1.2  Type fmtflags
-    // We use the bit values from libio.h if possible.
-    static const fmtflags boolalpha =   fmtflags(_IO_BOOLALPHA);
-    static const fmtflags dec =         fmtflags(_IO_DEC);
-    static const fmtflags fixed =       fmtflags(_IO_FIXED);
-    static const fmtflags hex =         fmtflags(_IO_HEX);
-    static const fmtflags internal =    fmtflags(_IO_INTERNAL);
-    static const fmtflags left =        fmtflags(_IO_LEFT);
-    static const fmtflags oct =         fmtflags(_IO_OCT);
-    static const fmtflags right =       fmtflags(_IO_RIGHT);
-    static const fmtflags scientific =  fmtflags(_IO_SCIENTIFIC);
-    static const fmtflags showbase =    fmtflags(_IO_SHOWBASE);
-    static const fmtflags showpoint =   fmtflags(_IO_SHOWPOINT);
-    static const fmtflags showpos =     fmtflags(_IO_SHOWPOS);
-    static const fmtflags skipws =      fmtflags(_IO_SKIPWS);
-    static const fmtflags unitbuf =     fmtflags(_IO_UNITBUF);
-    static const fmtflags uppercase =   fmtflags(_IO_UPPERCASE);
-    static const fmtflags adjustfield = fmtflags(_IO_LEFT | _IO_RIGHT |
-                                                 _IO_INTERNAL);
-    static const fmtflags basefield =   fmtflags(_IO_DEC | _IO_OCT | _IO_HEX);
-    static const fmtflags floatfield =  fmtflags(_IO_SCIENTIFIC | _IO_FIXED);
+    static const fmtflags boolalpha =   fmtflags(__ios_flags::_S_boolalpha);
+    static const fmtflags dec =         fmtflags(__ios_flags::_S_dec);
+    static const fmtflags fixed =       fmtflags(__ios_flags::_S_fixed);
+    static const fmtflags hex =         fmtflags(__ios_flags::_S_hex);
+    static const fmtflags internal =    fmtflags(__ios_flags::_S_internal);
+    static const fmtflags left =        fmtflags(__ios_flags::_S_left);
+    static const fmtflags oct =         fmtflags(__ios_flags::_S_oct);
+    static const fmtflags right =       fmtflags(__ios_flags::_S_right);
+    static const fmtflags scientific =  fmtflags(__ios_flags::_S_scientific);
+    static const fmtflags showbase =    fmtflags(__ios_flags::_S_showbase);
+    static const fmtflags showpoint =   fmtflags(__ios_flags::_S_showpoint);
+    static const fmtflags showpos =     fmtflags(__ios_flags::_S_showpos);
+    static const fmtflags skipws =      fmtflags(__ios_flags::_S_skipws);
+    static const fmtflags unitbuf =     fmtflags(__ios_flags::_S_unitbuf);
+    static const fmtflags uppercase =   fmtflags(__ios_flags::_S_uppercase);
+    static const fmtflags adjustfield = fmtflags(__ios_flags::_S_adjustfield);
+    static const fmtflags basefield =   fmtflags(__ios_flags::_S_basefield);
+    static const fmtflags floatfield =  fmtflags(__ios_flags::_S_floatfield);
 
     // 27.4.2.1.3  Type ios_base::iostate
     typedef _Ios_Iostate iostate;
-    static const iostate badbit =  iostate(_IO_BAD_SEEN);
-    static const iostate eofbit =  iostate(_IO_EOF_SEEN);
-    static const iostate failbit = iostate(_IO_ERR_SEEN);
-    static const iostate goodbit = iostate(0);
+    static const iostate badbit =  	iostate(__ios_flags::_S_badbit);
+    static const iostate eofbit =  	iostate(__ios_flags::_S_eofbit);
+    static const iostate failbit = 	iostate(__ios_flags::_S_failbit);
+    static const iostate goodbit = 	iostate(0);
 
     // 27.4.2.1.4  Type openmode
     typedef _Ios_Openmode openmode;
-    static const openmode app =    openmode(_IOS_APPEND);
-    static const openmode ate =    openmode(_IOS_ATEND);
-    static const openmode binary = openmode(_IOS_BIN);
-    static const openmode in =     openmode(_IOS_INPUT);
-    static const openmode out =    openmode(_IOS_OUTPUT);
-    static const openmode trunc =  openmode(_IOS_TRUNC);
+    static const openmode app =    	openmode(__ios_flags::_S_app);
+    static const openmode ate =    	openmode(__ios_flags::_S_ate);
+    static const openmode binary = 	openmode(__ios_flags::_S_bin);
+    static const openmode in =     	openmode(__ios_flags::_S_in);
+    static const openmode out =    	openmode(__ios_flags::_S_out);
+    static const openmode trunc =  	openmode(__ios_flags::_S_trunc);
 
     // 27.4.2.1.5  Type seekdir
     typedef _Ios_Seekdir seekdir;
-    static const seekdir beg = seekdir(0);
-    static const seekdir cur = seekdir(SEEK_CUR);
-    static const seekdir end = seekdir(SEEK_END);
+    static const seekdir beg = 		seekdir(0);
+    static const seekdir cur = 		seekdir(SEEK_CUR);
+    static const seekdir end = 		seekdir(SEEK_END);
 
 #ifdef _GLIBCPP_DEPRICATED
     typedef int io_state;
@@ -291,6 +289,7 @@ namespace std {
 
     class Init 
     {
+      friend class ios_base;
     public:
       Init();
       ~Init();
@@ -361,13 +360,7 @@ namespace std {
     }
 
     static bool 
-    sync_with_stdio(bool /*__sync*/ = true)
-    { 
-      // XXX Not done: 
-      // Otherwise, called with __sync == false, allows the standard
-      // streams to operate independently of the standard C streams.
-      return true; 
-    }
+    sync_with_stdio(bool __sync = true);
 
    // Locales:
     locale 
