@@ -1,6 +1,6 @@
 // File based streams -*- C++ -*-
 
-// Copyright (C) 1997-1999 Cygnus Solutions
+// Copyright (C) 1997-1999 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -38,12 +38,15 @@
 #include <bits/std_ostream.h>
 #include <bits/basic_file.h>
 #include <bits/std_locale.h>	// For codecvt
+#include <bits/c++threads.h>	// For __mutext_type
 
 namespace std {
 
   template<typename _CharT, typename _Traits>
     class basic_filebuf : public basic_streambuf<_CharT, _Traits>
     {
+    public:
+
       // Types:
       typedef _CharT                     	       	  char_type;
       typedef typename _Traits::int_type 		  int_type;
@@ -59,12 +62,16 @@ namespace std {
       typedef codecvt<_CharT, char, __state_type>         __codecvt_type;
       typedef codecvt<_CharT, char, __state_type>::result __res_type;
       
+      friend ios_base; // For sync_with_stdio.
+
+    private:
       // Data Members:
       __file_type* 		_M_file;
       bool			_M_last_overflowed;  // XXX Needed?
       __state_type		_M_state_cur;// Current state type for codecvt.
       __state_type 		_M_state_beg; 	
       const __codecvt_type*	_M_fcvt;       // Cached value from use_facet.
+      __mutext_type           	_M_lock;
 
     public:
       // Constructors/destructor:
@@ -86,30 +93,29 @@ namespace std {
 
       // Members:
       bool 
-      is_open() const { return _M_file->is_open(); }
+      is_open(void) const { return _M_file->is_open(); }
     
       __filebuf_type* 
       open(const char* __s, ios_base::openmode __mode);
     
       __filebuf_type* 
-      close();
+      close(void);
 
     protected:
-
       // Common initialization code for both ctors goes here.
       void
       _M_init_filebuf(void);
 
       // Overridden virtual functions:
       virtual streamsize 
-      showmanyc();
+      showmanyc(void);
    
       // Stroustrup, 1998, p. 628 
       // underflow() and uflow() functions are called to get the next
       // charater from the real input source when the buffer is empty.
       // Buffered input uses underflow()
       virtual int_type 
-      underflow();
+      underflow(void);
 
       virtual int_type 
       pbackfail(int_type __c = _Traits::eof());
@@ -150,25 +156,22 @@ namespace std {
 
       virtual pos_type 
       seekpos(pos_type __pos,
-	       ios_base::openmode __mode = ios_base::in | ios_base::out)
-      {
-	pos_type __retval =  this->seekoff(__pos, ios_base::beg, __mode); 
-	_M_last_overflowed = false;	
-	return __retval;
-      }
+	      ios_base::openmode __mode = ios_base::in | ios_base::out);
 
       virtual int 
-      sync()
+      sync(void)
       {
-	// Don't do anything about input sequences.
-	bool __testput = _M_out_cur && _M_out_beg < _M_out_cur;
+	bool __testput = _M_out_cur && _M_out_beg < _M_out_end;
 	if (__testput)
 	  {
-	    // Iff _M_mode is ios_base::out only, then need to set out
-	    // pointers to the end of the current put sequence before
-	    // we call _M_really_overflow, or else the stream will not be
-	    // flushed. Back up, so overflow doesn't append extra characters.
+	    // Need to restore current position. This interpreted as
+	    // the position of the external byte sequence (_M_file)
+	    // plus the offset in the current internal buffer
+	    // (_M_out_beg - _M_out_cur)
+	    __c_streampos __cur = _M_file->seekoff(0, ios_base::cur);
+	    off_type __off = _M_out_cur - _M_out_beg;
 	    this->_M_really_overflow();
+	    _M_file->seekpos(__cur + __off);
 	  }
 	_M_last_overflowed = false;	
 	return 0;
@@ -222,17 +225,17 @@ namespace std {
       { return static_cast<__filebuf_type*>(_M_streambuf); }
 
       bool 
-      is_open() { return rdbuf()->is_open(); }
+      is_open(void) { return rdbuf()->is_open(); }
 
       void 
       open(const char* __s, ios_base::openmode __mode = ios_base::in)
       { 
-	if (!rdbuf()->open(__s, __mode))
+	if (rdbuf()->open(__s, __mode | ios_base::in) == NULL)
 	  this->setstate(ios_base::failbit); 
       }
 
       void 
-      close()
+      close(void)
       { 
 	if (!rdbuf()->close())
 	  this->setstate(ios_base::failbit);	
@@ -274,22 +277,22 @@ namespace std {
 
       // Members:
       __filebuf_type* 
-      rdbuf() const
+      rdbuf(void) const
       { return static_cast<__filebuf_type*>(_M_streambuf); }
  
       bool 
-      is_open() { return rdbuf()->is_open(); }
+      is_open(void) { return rdbuf()->is_open(); }
 
       void 
       open(const char* __s, 
 	   ios_base::openmode __mode = ios_base::out | ios_base::trunc)
       { 
-	if (!rdbuf()->open(__s, __mode))
+	if (!rdbuf()->open(__s, __mode | ios_base::out))
 	  this->setstate (ios_base::failbit); 
       }
 
       void 
-      close()
+      close(void)
       { 
 	if (!rdbuf()->close())
 	  setstate (ios_base::failbit); 
@@ -336,11 +339,11 @@ namespace std {
     
       // Members:
       __filebuf_type* 
-      rdbuf() const 
+      rdbuf(void) const 
       { return static_cast<__filebuf_type*>(_M_streambuf); }
 
       bool 
-      is_open() { return rdbuf()->is_open(); }
+      is_open(void) { return rdbuf()->is_open(); }
 
       void 
       open(const char* __s, 
@@ -351,7 +354,7 @@ namespace std {
       }
 
       void 
-      close()
+      close(void)
       { 
 	if (!rdbuf()->close())
 	  setstate (ios_base::failbit); 
@@ -366,8 +369,15 @@ namespace std {
 
 #ifdef _GLIBCPP_NO_TEMPLATE_EXPORT
 # define export
+#ifdef  _GLIBCPP_FULLY_COMPLIANT_HEADERS
 # include <bits/fstream.tcc>
 #endif
+#endif
 
 #endif	/* _CPP_FSTREAM */
 
+
+
+
+
+
