@@ -1,6 +1,6 @@
 // The template and inlines for the -*- C++ -*- complex number classes.
 
-// Copyright (C) 1997-1999 Cygnus Solutions
+// Copyright (C) 1997-1999 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -27,9 +27,9 @@
 // invalidate any other reasons why the executable file might be covered by
 // the GNU General Public License.
 
-
-#include <bits/std_complex.h>
 #include <bits/std_cmath.h>
+#include <bits/std_complex.h>
+
 // This is a ISO C 9X header.
 #include <math/mathconf.h>
 #undef complex
@@ -42,33 +42,10 @@
 namespace std
 {
     
-    template<typename _Tp, typename _CharT, class _Traits>
-    basic_istream <_CharT, _Traits> &
-    operator>>(basic_istream<_CharT, _Traits>& __is, complex<_Tp>& __x)
-    {
-      //  XXX TODO
-      return __is;
-    }
-
-    template<typename _CharT, class _Traits>
-    basic_ostream<_CharT, _Traits>&
-    operator<<(basic_ostream<_CharT, _Traits>& __os, const complex<FLT>& __x)
-    {
-      basic_ostringstream<_CharT, _Traits> __s;
-      __s.flags(__os.flags());
-#ifdef _G_HAVE_LOCALE
-      __s.imbue(__os.getloc());
-#endif
-      __s.precision (__os.precision());
-      __s << '(' << __x.real() << "," << __x.imag() << ')' << ends;
-      return __os << __s.str();
-    }
-
-
     template<>
     FLT
     abs(const complex<FLT>& __x)
-    { return FCT(__mycabs)(__x._M_value); }
+    { return FCT(cabs)(__x._M_value); }
 
     template<>
     FLT
@@ -86,80 +63,200 @@ namespace std
       FCT(sincos)(__theta, &__sinx, &__cosx);
       return complex<FLT>(__rho * __cosx, __rho * __sinx);
 #else
-      return complex<FLT>(__rho * FCT(cos)(__theta), 
-			  __rho * FCT(sin)(__theta));
+#if defined(_GLIBCPP_BUGGY_FLOAT_COMPLEX) \
+      && defined(_GLIBCPP_FLOAT_SPECIALIZATION)
+      complex<FLT> __tmpf(__rho * FCT(cos)(__theta),
+                         __rho * FCT(sin)(__theta));
+      return __tmpf;
+#else
+       return complex<FLT>(__rho * FCT(cos)(__theta),
+                          __rho * FCT(sin)(__theta));
+#endif
 #endif
     }
 
     template<>
     complex<FLT>
     cos(const complex<FLT>& __x)
+#if defined(_GLIBCPP_BUGGY_FLOAT_COMPLEX) \
+      && defined(_GLIBCPP_FLOAT_SPECIALIZATION)
+    {
+      complex<FLT> __tmpf(FCT(ccos)(__x._M_value));
+      return __tmpf;
+    }
+#else
     { return complex<FLT>(FCT(ccos)(__x._M_value)); }
+#endif
 
     template<>
     complex<FLT>
     cosh(const complex<FLT>& __x)
+#if defined(_GLIBCPP_BUGGY_FLOAT_COMPLEX) \
+      && defined(_GLIBCPP_FLOAT_SPECIALIZATION)
+    {
+      complex<FLT> __tmpf(FCT(ccosh)(__x._M_value));
+      return __tmpf;
+    }
+#else
     { return complex<FLT>(FCT(ccosh)(__x._M_value)); }
+#endif
 
     template<>
     complex<FLT>
     exp(const complex<FLT>& __x)
+#if defined(_GLIBCPP_BUGGY_FLOAT_COMPLEX) \
+      && defined(_GLIBCPP_FLOAT_SPECIALIZATION)
+    {
+      complex<FLT> __tmpf(FCT(cexp)(__x._M_value));
+      return __tmpf;
+    }
+#else
     { return complex<FLT>(FCT(cexp)(__x._M_value)); }
+#endif
 
     template<>
     complex<FLT>
     log(const complex<FLT>& __x)
+#if defined(_GLIBCPP_BUGGY_FLOAT_COMPLEX) \
+      && defined(_GLIBCPP_FLOAT_SPECIALIZATION)
+    {
+      complex<FLT> __tmpf(FCT(c_log)(__x._M_value));
+      return __tmpf;
+    }
+#else
     { return complex<FLT>(FCT(c_log)(__x._M_value)); }
+#endif
 
     template<>
     complex<FLT>
     log10(const complex<FLT>& __x)
+#if defined(_GLIBCPP_BUGGY_FLOAT_COMPLEX) \
+      && defined(_GLIBCPP_FLOAT_SPECIALIZATION)
+    {
+      complex<FLT> __tmpf(FCT(clog10)(__x._M_value));
+      return __tmpf;
+    }
+#else
     { return complex<FLT>(FCT(clog10)(__x._M_value)); }
+#endif
 
     template<>
     complex<FLT>
     pow(const complex<FLT>& __x, int __n)
+#if defined(_GLIBCPP_BUGGY_FLOAT_COMPLEX) \
+      && defined(_GLIBCPP_FLOAT_SPECIALIZATION)
+    {
+      complex<FLT> __tmpf(FCT(cexp) (__n * FCT(c_log)(__x._M_value)));
+      return __tmpf;
+    }
+#else
     { return complex<FLT>(FCT(cexp) (__n * FCT(c_log)(__x._M_value))); }
+#endif
+
 
     template<>
     complex<FLT>
     pow(const complex<FLT>& __x, const FLT& __y)
+#if defined(_GLIBCPP_BUGGY_FLOAT_COMPLEX) \
+      && defined(_GLIBCPP_FLOAT_SPECIALIZATION)
+    {
+      complex<FLT> __tmpf(FCT(cexp) (__y * FCT(c_log)(__x._M_value)));
+      return __tmpf;
+    }
+#else
     { return complex<FLT>(FCT(cexp) (__y * FCT(c_log)(__x._M_value))); }
+#endif
 
     template<>
     complex<FLT>
     pow(const complex<FLT>& __x, const complex<FLT>& __y)
+#if defined(_GLIBCPP_BUGGY_FLOAT_COMPLEX) \
+      && defined(_GLIBCPP_FLOAT_SPECIALIZATION)
+    {
+      complex<FLT> __tmpf(FCT(cpow)(__x._M_value, __y._M_value));
+      return __tmpf;
+    }
+#else
     { return complex<FLT>(FCT(cpow)(__x._M_value, __y._M_value)); }
+#endif
 
     template<>
     complex<FLT>
     pow(const FLT& __x, const complex<FLT>& __y)
+#if defined(_GLIBCPP_BUGGY_FLOAT_COMPLEX) \
+      && defined(_GLIBCPP_FLOAT_SPECIALIZATION)
+    {
+      complex<FLT> __tmpf(FCT(cexp)(__y._M_value * FCT(log)(__x)));
+      return __tmpf;
+    }
+#else
     { return complex<FLT>(FCT(cexp)(__y._M_value * FCT(log)(__x))); }
+#endif
 
     template<>
     complex<FLT>
     sin(const complex<FLT>& __x)
+#if defined(_GLIBCPP_BUGGY_FLOAT_COMPLEX) \
+      && defined(_GLIBCPP_FLOAT_SPECIALIZATION)
+    {
+      complex<FLT> __tmpf(FCT(csin)(__x._M_value));
+      return __tmpf;
+    }
+#else
     { return complex<FLT>(FCT(csin)(__x._M_value)); }
+#endif
 
     template<>
     complex<FLT>
     sinh(const complex<FLT>& __x)
+#if defined(_GLIBCPP_BUGGY_FLOAT_COMPLEX) \
+      && defined(_GLIBCPP_FLOAT_SPECIALIZATION)
+    {
+      complex<FLT> __tmpf(FCT(csinh)(__x._M_value));
+      return __tmpf;
+    }
+#else
     { return complex<FLT>(FCT(csinh)(__x._M_value)); }
+#endif
 
     template<>
     complex<FLT>
     sqrt(const complex<FLT>& __x)
+#if defined(_GLIBCPP_BUGGY_FLOAT_COMPLEX) \
+      && defined(_GLIBCPP_FLOAT_SPECIALIZATION)
+    {
+      complex<FLT> __tmpf(FCT(csqrt)(__x._M_value));
+      return __tmpf;
+    }
+#else
     { return complex<FLT>(FCT(csqrt)(__x._M_value)); }
+#endif
 
     template<>
     complex<FLT>
     tan(const complex<FLT>& __x)
-    { return complex<FLT>(FCT(ctan)(__x._M_value)); }
+#if defined(_GLIBCPP_BUGGY_FLOAT_COMPLEX) \
+      && defined(_GLIBCPP_FLOAT_SPECIALIZATION)
+    {
+      complex<FLT> __tmpf(FCT(ctan)(__x._M_value));
+      return __tmpf;
+    }
+#else
+     { return complex<FLT>(FCT(ctan)(__x._M_value)); }
+#endif
 
     template<>
     complex<FLT>
     tanh(const complex<FLT>& __x)
-    { return complex<FLT>(FCT(ctanh)(__x._M_value)); }
+#if defined(_GLIBCPP_BUGGY_FLOAT_COMPLEX) \
+      && defined(_GLIBCPP_FLOAT_SPECIALIZATION)
+    {
+      complex<FLT> __tmpf(FCT(ctanh)(__x._M_value));
+      return __tmpf;
+    }
+#else
+     { return complex<FLT>(FCT(ctanh)(__x._M_value)); }
+#endif
     
 } // namespace std
 
