@@ -1,6 +1,6 @@
 // Iostreams base classes -*- C++ -*-
 
-// Copyright (C) 1997-1999 Cygnus Solutions
+// Copyright (C) 1997-1999 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -252,6 +252,46 @@ namespace std {
     _M_callbacks = 0;
   }
 
+  bool 
+  ios_base::sync_with_stdio(bool __sync)
+  { 
+#ifdef _GLIBCPP_RESOLVE_LIB_DEFECTS
+    // 49.  Underspecification of ios_base::sync_with_stdio
+    bool __retval = __ioinit._M_cin->_M_file->get_fileno() == 0;
+    
+    // Turn off sync with C FILE* for cin, cout, cerr, clog.
+    if (!__sync && __retval)
+      {
+	__ioinit._M_cout = new filebuf();
+	__ioinit._M_cout->open("stdout", ios_base::out);
+	__ioinit._M_cin = new filebuf();
+	__ioinit._M_cin->open("stdin", ios_base::in);
+	__ioinit._M_cerr = new filebuf();
+	__ioinit._M_cerr->open("stderr", ios_base::out);
+	cout.rdbuf(__ioinit._M_cout);
+	cin.rdbuf(__ioinit._M_cin);
+	cerr.rdbuf(__ioinit._M_cerr);
+	cerr.flags(ios_base::unitbuf);
+	clog.rdbuf(__ioinit._M_cerr);
+#ifdef _GLIBCPP_USE_WCHAR_T
+	__ioinit._M_wcout = new wfilebuf();
+	__ioinit._M_wcout->open("stdout", ios_base::out);
+	__ioinit._M_wcin = new wfilebuf();
+	__ioinit._M_wcin->open("stdin", ios_base::in);
+	__ioinit._M_wcerr = new wfilebuf();
+	__ioinit._M_wcerr->open("stderr", ios_base::out);
+	wcout.rdbuf(__ioinit._M_wcout);
+	wcin.rdbuf(__ioinit._M_wcin);
+	wcerr.rdbuf(__ioinit._M_wcerr);
+	wcerr.flags(ios_base::unitbuf);
+	wclog.rdbuf(__ioinit._M_wcerr);
+#endif
+      }
+    
+    return __retval; 
+#endif
+  }
+
 }  // namespace std
 
 
@@ -262,3 +302,6 @@ namespace std {
 
 
 
+
+
+
