@@ -1,4 +1,4 @@
-// Copyright (C) 1997-1999 Cygnus Solutions
+// Copyright (C) 1997, 1998, 1999, 2000 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -36,6 +36,7 @@
 #include <bits/std_istream.h>
 #include <bits/std_ostream.h>
 #include <bits/std_vector.h>
+#include <bits/std_memory.h>      // for auto_ptr
 
 namespace std {
 
@@ -51,58 +52,46 @@ namespace std {
 
   locale::_Impl::_Impl(size_t __numfacets, size_t __refs)
   : _M_num_references(__refs - 1)
-    //  , _M_facets (numfacets, (facet*)0)
-    //  , _M_category_names (_S_num_categories, string ("*"))
+  , _M_facets(0)
+  , _M_category_names(0)
   , _M_has_name(false)
   , _M_cached_name_ok(false)
   , _M_cached_name(string ("*"))
   { 
     typedef vector<facet*, __STL_DEFAULT_ALLOCATOR(facet*) > __vec_facet;
     typedef vector<string, __STL_DEFAULT_ALLOCATOR(string) > __vec_string;
-    try {
-      _M_facets = new __vec_facet(__numfacets, (facet*)0);
-    }
-    catch (...) {
-      delete _M_facets;
-      throw;
-    }
-    try {
-       _M_category_names = new __vec_string(_S_num_categories, string("*"));
-    }
-    catch (...) {
-      delete _M_category_names;
-      throw;
-    }
+
+    auto_ptr<__vec_facet> __pvf(new __vec_facet(__numfacets, (facet*)0));
+    auto_ptr<__vec_string> __pcn(new __vec_string(_S_num_categories,
+						  string("*")));
+    _M_facets = __pvf.release();
+    _M_category_names = __pcn.release();
   }
   
   locale::_Impl::_Impl(const _Impl& __other, size_t __refs)
   : _M_num_references(__refs)
-    //  , _M_facets (other._M_facets)
-    //  , _M_category_names (other._M_category_names)
+  , _M_facets(0)
+  , _M_category_names(0)
   , _M_has_name(__other._M_has_name)
   , _M_cached_name_ok(__other._M_cached_name_ok)
   , _M_cached_name(__other._M_cached_name)
   {
     typedef vector<facet*, __STL_DEFAULT_ALLOCATOR(facet*) > __vec_facet;
     typedef vector<string, __STL_DEFAULT_ALLOCATOR(string) > __vec_string;
-    try {
-      _M_facets = new __vec_facet(*(__other._M_facets));
-    }
-    catch (...) {
-      delete _M_facets;
-      throw;
-    }
-    try {
-       _M_category_names = new __vec_string(*(__other._M_category_names));
-    }
-    catch (...) {
-      delete _M_category_names;
-      throw;
-    }
 
-    std::vector<facet*>::iterator __it = _M_facets->begin();
-    for (; __it != _M_facets->end(); ++__it)
+    auto_ptr<__vec_facet> __pvf(new __vec_facet(*(__other._M_facets)));
+    auto_ptr<__vec_string> 
+      __pcn(new __vec_string(*(__other._M_category_names)));
+
+    std::vector<facet*>::iterator __it = __pvf->begin();
+    for (; __it != __pvf->end(); ++__it)
       (*__it)->_M_add_reference();
+
+    // these must be last since in the presence of an exception, the 
+    // destructor for 'this' won't run until AFTER execution has passed  
+    // the closing brace of the constructor
+    _M_facets = __pvf.release();
+    _M_category_names = __pcn.release();
   }
   
   void
@@ -302,7 +291,6 @@ namespace std {
     return *this;
   }
 
-  ///////
   locale
   locale::global(const locale& __other)
   {
@@ -317,7 +305,6 @@ namespace std {
     return __keep;
   }
 
-  /////////
   string
   locale::name() const
   {
@@ -325,7 +312,6 @@ namespace std {
     return "*";
   }
 
-  ///////
   locale const&
   locale::classic()
   {
@@ -411,7 +397,7 @@ namespace std {
       case LC_MONETARY: return monetary;
       case LC_NUMERIC:  return numeric;
       case LC_TIME:     return time; 
-#ifdef HAVE_LC_MESSAGES
+#ifdef _GLIBCPP_HAVE_LC_MESSAGES
       case LC_MESSAGES: return messages;
 #endif	
       case LC_ALL:      return all;
@@ -454,61 +440,18 @@ namespace std {
   
   size_t locale::id::_S_highwater;  // init'd to 0 by linker
 
+
+  // Platform-specific initialization code for ctype tables.
+  #include <ctype.cc>
+
   locale::id ctype<char>::id;
 
   ctype<char>::~ctype()
-  { if (_M_del) delete[] table (); }
-
-#if defined(__GLIBC__) && (__GLIBC__ >= 2) && defined(_GLIBCPP_USE_CTYPE_ISBIT)
-  // XXX these are Glibc-specific.
-  const int* const& ctype<char>::_S_toupper = __ctype_toupper;
-  const int* const& ctype<char>::_S_tolower = __ctype_tolower;
-  const ctype_base::mask* const& ctype<char>::_S_table = __ctype_b;
-#else
-  const int* const& ctype<char>::_S_toupper = 0;
-  const int* const& ctype<char>::_S_tolower = 0;
-  const ctype_base::mask* const& ctype<char>::_S_table = 0;
-#endif
-  
-  char
-  ctype<char>::do_toupper(char __c) const
-  {
-    return _S_toupper[(int) __c];
-  }
-
-  const char*
-  ctype<char>::do_toupper(char* __low, const char* __high) const
-  {
-    while (__low < __high)
-      {
-	*__low = _S_toupper[(int) *__low];
-	++__low;
-      }
-    return __high;
-  }
-
-  char
-  ctype<char>::do_tolower(char __c) const
-  {
-    return _S_tolower[(int) __c];
-  }
-
-  const char* 
-  ctype<char>::do_tolower(char* __low, const char* __high) const
-  {
-    while (__low < __high)
-      {
-	*__low = _S_tolower[(int) *__low];
-	++__low;
-      }
-    return __high;
-  }
+  { if (_M_del) delete[] table(); }
 
   char
   ctype<char>::do_widen(char __c) const
-  {
-    return __c;
-  }
+  { return __c; }
   
   const char* 
   ctype<char>::
@@ -519,25 +462,21 @@ namespace std {
   }
   
   char
-  ctype<char>::do_narrow(char __c, char __dfault) const
-  {
-    return __c;
-  }
+  ctype<char>::do_narrow(char __c, char /*__dfault*/) const
+  { return __c; }
   
   const char* 
   ctype<char>::
-  do_narrow(const char* __low, const char* __high, char __dfault,
+  do_narrow(const char* __low, const char* __high, char /*__dfault*/,
 	    char* __dest) const
   {
     memcpy(__dest, __low, __high - __low);
     return __high;
   }
 
-  template<>
-    ctype_byname<char>::ctype_byname(const char* __s, size_t __refs)
-    : ctype<char> (new mask[table_size], true, __refs)
-    { }
-  
+  ctype_byname<char>::ctype_byname(const char* /*__s*/, size_t __refs)
+  : ctype<char> (new mask[table_size], true, __refs)
+  { }
 
   locale::id codecvt<char, char, mbstate_t>::id;
 
@@ -549,7 +488,7 @@ namespace std {
   
   codecvt_base::result
   codecvt<char, char, mbstate_t>::
-  do_out(state_type& __state, const intern_type* __from, 
+  do_out(state_type& /*__state*/, const intern_type* __from, 
 	 const intern_type* __from_end, const intern_type*& __from_next,
 	 extern_type* __to, extern_type* __to_end, 
 	 extern_type*& __to_next) const
@@ -565,7 +504,8 @@ namespace std {
   
   codecvt_base::result
   codecvt<char, char, mbstate_t>::
-  do_unshift(state_type& __state, extern_type* __to, extern_type* __to_limit,
+  do_unshift(state_type& /*__state*/, extern_type* __to,
+             extern_type* /*__to_limit*/,
 	     extern_type*& __to_next) const
   { 
     __to_next = __to; 
@@ -574,7 +514,7 @@ namespace std {
   
   codecvt_base::result
   codecvt<char, char, mbstate_t>::
-  do_in(state_type& __state, const extern_type* __from, 
+  do_in(state_type& /*__state*/, const extern_type* __from, 
 	const extern_type* __from_end, const extern_type*& __from_next,
 	intern_type* __to, intern_type* __to_end, 
 	intern_type*& __to_next) const
@@ -599,7 +539,7 @@ namespace std {
   
   int 
   codecvt<char, char, mbstate_t>::
-  do_length (const state_type& __state, const extern_type* __from,
+  do_length (const state_type& /*__state*/, const extern_type* __from,
 	     const extern_type* __end, size_t __max) const
   { return (__max < size_t(__end - __from)) ? __max : __end - __from; }
   
@@ -608,7 +548,7 @@ namespace std {
   { return 1; }
   
   codecvt_byname<char, char, mbstate_t>::
-  codecvt_byname(const char* __s, size_t __refs)
+  codecvt_byname(const char* /*__s*/, size_t __refs)
   : codecvt<char, char, mbstate_t>(__refs)
   { }
 
@@ -639,9 +579,7 @@ namespace std {
   
   string
   collate<char>::do_transform(const char* __lo, const char* __hi) const
-  {
-    return string(__lo, __hi - __lo);
-  }
+  { return string(__lo, __hi - __lo); }
   
   long
   collate<char>::do_hash(const char* __lo, const char* __hi) const
@@ -653,58 +591,38 @@ namespace std {
     return __val;
   }
   
-  template<>
-    collate_byname<char>::collate_byname(const char* __s, size_t __refs)
-    : collate<char> (__refs)
-    { }
+  collate_byname<char>::collate_byname(const char* /*__s*/, size_t __refs)
+  : collate<char> (__refs)
+  { }
 
-  template<>
-    numpunct_byname<char>::numpunct_byname(const char* __s, size_t __refs)
-    : numpunct<char> (__refs)
-    { }
+  numpunct_byname<char>::numpunct_byname(const char* /*__s*/, size_t __refs)
+  : numpunct<char> (__refs)
+  { }
 
-
-  template<>
-    moneypunct_byname<char, false>::
-    moneypunct_byname(const char* __s, size_t __refs)
-    : moneypunct<char, false> (__refs)
-    { }
+  moneypunct_byname<char, false>::
+  moneypunct_byname(const char* /*__s*/, size_t __refs)
+  : moneypunct<char, false> (__refs)
+  { }
   
-  template<>
-    moneypunct_byname<char, true>::
-    moneypunct_byname(const char* __s, size_t __refs)
-    : moneypunct<char, true> (__refs)
-    { }
+  moneypunct_byname<char, true>::
+  moneypunct_byname(const char* /*__s*/, size_t __refs)
+  : moneypunct<char, true> (__refs)
+  { }
   
-  template<>
-    messages_byname<char>::messages_byname(const char* __s, size_t __refs)
-    : messages<char> (__refs)
-    { }
+  messages_byname<char>::messages_byname(const char* /*__s*/, size_t __refs)
+  : messages<char> (__refs)
+  { }
 
 #ifdef _GLIBCPP_USE_WCHAR_T  
   locale::id ctype<wchar_t>::id;
 
-#if defined(__GLIBC__) && (__GLIBC__ >= 2) && defined(_GLIBCPP_USE_CTYPE_ISBIT)
-  // XXX These are Glibc-specific.
-  const int* const& ctype<wchar_t>::_S_toupper = __ctype_toupper;
-  const int* const& ctype<wchar_t>::_S_tolower = __ctype_tolower;
-  const ctype_base::mask* const& ctype<wchar_t>::_S_table = __ctype_b;
-#else
-  const int* const& ctype<wchar_t>::_S_toupper = 0;
-  const int* const& ctype<wchar_t>::_S_tolower = 0;
-  const ctype_base::mask* const& ctype<wchar_t>::_S_table = 0;
-#endif
-
-  ctype<wchar_t>::ctype(size_t __refs)
-  : _Ctype<wchar_t>(__refs)
-  { }
-
   ctype<wchar_t>::~ctype() { }
 
   bool
   ctype<wchar_t>::do_is(mask __m, char_type __c) const
-  {
-    return (__c < _S_table_size) && (_S_table[__c] & __m); 
+  { 
+    return ((static_cast<__table_type>(__c) < _S_table_size) 
+	    && (_M_ctable[__c] & __m)); 
   }
   
   const wchar_t* 
@@ -712,7 +630,8 @@ namespace std {
 			mask* __vec) const
   {
     for (; __low < __high; ++__low, ++__vec)
-      *__vec = (*__low < _S_table_size) ? _S_table[*__low] : mask(0);
+      *__vec = ((static_cast<__table_type>(*__low) < _S_table_size) 
+		? _M_ctable[*__low] : mask(0));
     return __high;
   }
   
@@ -721,7 +640,8 @@ namespace std {
 			     const wchar_t* __high) const
   {
     while (__low < __high 
-	   && (_S_table_size < *__low || !(_S_table[*__low] & __m)))
+	   && (_S_table_size < static_cast<__table_type>(*__low) 
+	       || !(_M_ctable[*__low] & __m)))
       ++__low;
     return __low;
   }
@@ -731,61 +651,30 @@ namespace std {
 			      const char_type* __high) const
   {
     while (__low < __high 
-	   && *__low < _S_table_size 
-	   && (_S_table[*__low] & __m))
+	   && static_cast<__table_type>(*__low) < _S_table_size 
+	   && (_M_ctable[*__low] & __m))
       ++__low;
     return __low;
   }
 
-  wchar_t
-  ctype<wchar_t>::do_toupper(char_type __c) const
-  {
-    return (__c < _S_table_size) ? _S_toupper[__c] : __c;
-  }
-  
-  const wchar_t*
-  ctype<wchar_t>::do_toupper(char_type* low, const char_type* high) const
-  {
-    for (;low < high; ++low)
-      if (*low < _S_table_size)
-        *low = _S_toupper[*low];
-    return high;
-  }
-  
-  wchar_t
-  ctype<wchar_t>::do_tolower(char_type __c) const
-  {
-    return (__c < _S_table_size) ? _S_tolower[__c] : __c;
-  }
-  
-  const wchar_t*
-  ctype<wchar_t>::do_tolower(char_type* __low, const char_type* __high) const
-  {
-    for (; __low < __high; ++__low)
-      if (*__low < _S_table_size)
-        *__low = _S_toupper[*__low];
-    return __high;
-  }
-
   wchar_t
   ctype<wchar_t>::do_widen(char __c) const
-  {
-    return static_cast<wchar_t> ((unsigned char)__c);
-  }
+  { return static_cast<wchar_t>((unsigned char)__c); }
   
   const char* 
   ctype<wchar_t>::do_widen(const char* __low, const char* __high,
 			      wchar_t* __dest) const
   {
     while (__low < __high)
-      *__dest++ = static_cast<wchar_t> ((unsigned char)*__low++);
+      *__dest++ = static_cast<wchar_t>((unsigned char)*__low++);
     return __high;
   }
 
   char
   ctype<wchar_t>::do_narrow(wchar_t __c, char __dfault) const
-  {
-    return (__c < _S_table_size) ? static_cast<char> (__c) : __dfault;
+  { 
+    return ((static_cast<__table_type>(__c) < _S_table_size) 
+	    ? static_cast<char>(__c) : __dfault); 
   }
 
   const wchar_t*
@@ -793,14 +682,14 @@ namespace std {
 			    char __dfault, char* __dest) const
   {
     for (; __low < __high; ++__dest, ++__low)
-      *__dest = (*__low < _S_table_size) ? static_cast<char> (*__low) : __dfault;
+      *__dest = (static_cast<__table_type>(*__low) < _S_table_size) 
+		? static_cast<char>(*__low) : __dfault;
     return __high;
   }
 
-  template<>
-    ctype_byname<wchar_t>::ctype_byname(const char* __s, size_t __refs)
-    : ctype<wchar_t> (__refs)
-    { }
+  ctype_byname<wchar_t>::ctype_byname(const char* /*__s*/, size_t __refs)
+  : ctype<wchar_t> (__refs)
+  { }
 
   locale::id codecvt<wchar_t, char, mbstate_t>::id;
 
@@ -812,7 +701,7 @@ namespace std {
   
   codecvt_base::result
   codecvt<wchar_t, char, mbstate_t>::
-  do_out(state_type& __state, const intern_type* __from, 
+  do_out(state_type& /*__state*/, const intern_type* __from, 
 	 const intern_type* __from_end, const intern_type*& __from_next,
 	 extern_type* __to, extern_type* __to_limit,
 	 extern_type*& __to_next) const
@@ -825,7 +714,8 @@ namespace std {
   
   codecvt_base::result
   codecvt<wchar_t, char, mbstate_t>::
-  do_unshift (state_type& __state, extern_type* __to, extern_type* __to_limit,
+  do_unshift (state_type& /*__state*/, extern_type* __to,
+              extern_type* /*__to_limit*/,
 	      extern_type*& __to_next) const
   {
     __to_next = __to;
@@ -834,13 +724,13 @@ namespace std {
   
   codecvt_base::result
   codecvt<wchar_t, char, mbstate_t>::
-  do_in (state_type& __state, const extern_type* __from, 
-	 const extern_type* __from_end, const extern_type*& __from_next,
-	 intern_type* __to, intern_type* __to_limit,
-	 intern_type*& __to_next) const
+  do_in(state_type& /*__state*/, const extern_type* __from, 
+	const extern_type* __from_end, const extern_type*& __from_next,
+	intern_type* __to, intern_type* __to_limit,
+	intern_type*& __to_next) const
   {
     for (; __from < __from_end && __to < __to_limit; ++__from, ++__to)
-      *__to = static_cast<wchar_t> (*__from);
+      *__to = static_cast<wchar_t>(*__from);
     __from_next = __from; 
     __to_next = __to;
     return __from == __from_end ? ok : partial;
@@ -855,7 +745,7 @@ namespace std {
   
   int 
   codecvt<wchar_t, char, mbstate_t>::
-  do_length(const state_type& __state, const extern_type* __from,
+  do_length(const state_type& /*__state*/, const extern_type* __from,
 	    const extern_type* __end, size_t __max) const
   { return (__max < size_t(__end - __from)) ? __max : __end - __from; }
   
@@ -864,7 +754,7 @@ namespace std {
   { return 1; }
 
   codecvt_byname<wchar_t, char, mbstate_t>::
-  codecvt_byname(const char* __s, size_t __refs)
+  codecvt_byname(const char* /*__s*/, size_t __refs)
   : codecvt<wchar_t, char, mbstate_t> (__refs)
   {
   }
@@ -880,52 +770,51 @@ namespace std {
   collate<wchar_t>::~collate() { }
 
   int collate<wchar_t>::
-  do_compare(const wchar_t* __lo1, const wchar_t* __hi1,
-	     const wchar_t* __lo2, const wchar_t* __hi2) const
+  do_compare(const wchar_t* /*__lo1*/, const wchar_t* /*__hi1*/,
+	     const wchar_t* /*__lo2*/, const wchar_t* /*__hi2*/) const
   {
     return 0; // XXX not done
   }
   
-  wstring collate<wchar_t>::do_transform(const wchar_t* __lo, 
-					 const wchar_t* __hi) const
+  wstring collate<wchar_t>::do_transform(const wchar_t* /*__lo*/, 
+					 const wchar_t* /*__hi*/) const
   {
     return wstring(); // XXX not done
   }
   
   long collate<wchar_t>::
-  do_hash(const wchar_t* __lo, const wchar_t* __hi) const
+  do_hash(const wchar_t* /*__lo*/, const wchar_t* /*__hi*/) const
   {
     return 0; // XXX not done
   }
 
-  template<>
-    numpunct_byname<wchar_t>::numpunct_byname(const char* __s, size_t __refs)
-    : numpunct<wchar_t> (__refs)
-    { }
+  numpunct_byname<wchar_t>::numpunct_byname(const char* /*__s*/, size_t __refs)
+  : numpunct<wchar_t> (__refs)
+  { }
 
-  template<>
-    collate_byname<wchar_t>::collate_byname(const char* __s, size_t __refs)
-    : collate<wchar_t> (__refs)
-    { }
+  collate_byname<wchar_t>::collate_byname(const char* /*__s*/, size_t __refs)
+  : collate<wchar_t> (__refs)
+  { }
   
-  template<>
-    moneypunct_byname<wchar_t, false>::
-    moneypunct_byname(const char* __s, size_t __refs)
-    : moneypunct<wchar_t, false> (__refs)
-    { }
+  moneypunct_byname<wchar_t, false>::
+  moneypunct_byname(const char* /*__s*/, size_t __refs)
+  : moneypunct<wchar_t, false> (__refs)
+  { }
   
-  template<>
-    moneypunct_byname<wchar_t, true>::
-    moneypunct_byname(const char* __s, size_t __refs)
-    : moneypunct<wchar_t, true> (__refs)
-    { }
+  moneypunct_byname<wchar_t, true>::
+  moneypunct_byname(const char* /*__s*/, size_t __refs)
+  : moneypunct<wchar_t, true> (__refs)
+  { }
     
-  template<>
-    messages_byname<wchar_t>::messages_byname(const char* __s, size_t __refs)
-    : messages<wchar_t> (__refs)
-    { }
+  messages_byname<wchar_t>::messages_byname(const char* /*__s*/, size_t __refs)
+  : messages<wchar_t> (__refs)
+  { }
 #endif //  _GLIBCPP_USE_WCHAR_T
 
 } // namespace std
 
 
+
+
+
+
