@@ -38,10 +38,9 @@
 #include <bits/std_cstddef.h>     // for size_t
 #include <bits/std_string.h>
 #include <bits/std_stdexcept.h>
-
 #include <bits/std_istream.h>
 #include <bits/std_ostream.h>
-
+#include <bits/std_algorithm.h>
 
 #define __BITS_PER_WORDT(__wt) (CHAR_BIT*sizeof(__wt))
 #define __BITSET_WORDS(__n,__wt) \
@@ -196,10 +195,13 @@ void _Base_bitset<_Nw, _WordT>::_M_do_left_shift(size_t __shift)
     const size_t __wshift = __shift / __BITS_PER_WORDT(_WordT);
     const size_t __offset = __shift % __BITS_PER_WORDT(_WordT);
     const size_t __sub_offset = __BITS_PER_WORDT(_WordT) - __offset;
+    const _WordT __mask = __offset == static_cast<size_t>(0) ?
+			  static_cast<_WordT>(0) :
+			  ~static_cast<_WordT>(0);
     size_t __n = _Nw - 1;
     for ( ; __n > __wshift; --__n)
       _M_w[__n] = (_M_w[__n - __wshift] << __offset) |
-                (_M_w[__n - __wshift - 1] >> __sub_offset);
+		((_M_w[__n - __wshift - 1] >> __sub_offset) & __mask);
     if (__n == __wshift)
       _M_w[__n] = _M_w[0] << __offset;
     for (size_t __n1 = 0; __n1 < __n; ++__n1)
@@ -214,11 +216,14 @@ void _Base_bitset<_Nw, _WordT>::_M_do_right_shift(size_t __shift)
     const size_t __wshift = __shift / __BITS_PER_WORDT(_WordT);
     const size_t __offset = __shift % __BITS_PER_WORDT(_WordT);
     const size_t __sub_offset = __BITS_PER_WORDT(_WordT) - __offset;
+    const _WordT __mask = __offset == static_cast<size_t>(0) ?
+			  static_cast<_WordT>(0) :
+			  ~static_cast<_WordT>(0);
     const size_t __limit = _Nw - __wshift - 1;
     size_t __n = 0;
     for ( ; __n < __limit; ++__n)
       _M_w[__n] = (_M_w[__n + __wshift] >> __offset) |
-                  (_M_w[__n + __wshift + 1] << __sub_offset);
+		  ((_M_w[__n + __wshift + 1] << __sub_offset) & __mask);
     _M_w[__limit] = _M_w[_Nw-1] >> __offset;
     for (size_t __n1 = __limit + 1; __n1 < _Nw; ++__n1)
       _M_w[__n1] = static_cast<_WordT>(0);
@@ -557,7 +562,6 @@ private:
   // name resolution rules.
 
 #ifdef __STL_HAS_NAMESPACES
-
   using _Base::_S_whichword;
   using _Base::_S_whichbyte;
   using _Base::_S_whichbit;
@@ -578,7 +582,6 @@ private:
   using _Base::_M_do_to_ulong;
   using _Base::_M_do_find_first;
   using _Base::_M_do_find_next;
-
 #endif /* __STL_HAS_NAMESPACES */
 
 private:
@@ -648,19 +651,7 @@ public:
   bitset(unsigned long __val) :
     _Base_bitset<__BITSET_WORDS(_Nb,_WordT), _WordT>(__val) {}
 
-  template<class _CharT, class _Traits, class _Alloc>
-  explicit bitset(const basic_string<_CharT,_Traits,_Alloc>& __s,
-                  size_t __pos = 0, 
-		  size_t __n = __s.npos) 
-    : _Base()
-  {
-    if (__pos > __s.size())
-      __STL_THROW(out_of_range("bitset"));
-    _M_copy_from_string(__s, __pos, __n);
-  }
-
 #ifdef __STL_MEMBER_TEMPLATES
-
   template<class _CharT, class _Traits, class _Alloc>
   explicit bitset(const basic_string<_CharT,_Traits,_Alloc>& __s,
                   size_t __pos = 0)
@@ -682,9 +673,7 @@ public:
 	__STL_THROW(out_of_range("bitset"));
       _M_copy_from_string(__s, __pos, __n);
     }
-
-#else /* __STL_MEMBER_TEMPLATES */
-
+#else 
   explicit bitset(const basic_string<char>& __s,
 		  size_t __pos = 0,
 		  size_t __n = basic_string<char>::npos) 
